/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.File;
import java.io.IOException;

import org.moeaframework.Executor;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.PRNG;
import org.moeaframework.core.PopulationIO;
import org.moeaframework.analysis.plot.Plot;


/**
 * Demonstrates using an Executor to solve the UF1 test problem with NSGA-II,
 * one of the most widely-used multiobjective evolutionary algorithms.
 */
public class Example1 {

	public static void main(String[] args) throws IOException {
		//configure and run this experiment
		PRNG.setSeed(5);
		NondominatedPopulation result = new Executor()
				.withProblem("dtlz2_3")
				.withProperty("populationSize", 100)
				.withAlgorithm("DI-1")
				.withMaxEvaluations(100000)
				.run();
		
		//display the results
//		System.out.format("Objective1  Objective2   %n");
//		
//		for (Solution solution : result) {
//			System.out.format("%.4f     %.4f %n",
//					solution.getObjective(0),
//					solution.getObjective(1));
//		}
//		
       new Plot()
              .add("hsms1e", result)
              .show();
		
       
       PopulationIO.writeObjectives(new File("output1e.txt"), result);
	}

}
