/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.io.IOException;

import org.moeaframework.Analyzer;
import org.moeaframework.Executor;
import org.moeaframework.core.PRNG;

/**
 * Demonstrates the use of the {@code Instrumenter} to collect run-time
 * dynamics.  The output lists the NFE, elapsed time and generational distance
 * throughout the run.
 */
public class Example3 {

	public static void main(String[] args) throws IOException {
		// setup the instrumenter to record the generational distance metric
		String problem = "dtlz7_3";
		//String[] algorithms = { "NSGA2", "NSGA3", "MOEAD"};
		String[] algorithms = {"SMS-EMOA","DI-1", "DI-2","NSGA2", "NSGA3","MOEAD"};

//		Instrumenter instrumenter = new Instrumenter()
//				.withProblem(problem)
//				.withFrequency(50000)
//				.attachElapsedTimeCollector()
//				.attachGenerationalDistanceCollector()
//				.attachAdaptiveTimeContinuationCollector()
//				.attachHypervolumeCollector();
				//.attachAll();

		// use the executor to run the algorithm with the instrumenter
		Executor executor =new Executor()
				.withProblem(problem)
				//.withAlgorithm("NSGAII")
				.withProperty("populationSize", 100)
				.withMaxEvaluations(100000)
				//.withInstrumenter(instrumenter)
				.distributeOnAllCores();
				//.run();
		
//		double[] minBound={0, 0};
//		double[] maxBound={1.1, 1.1};
		
		Analyzer analyzer = new Analyzer()
				.withProblem(problem)
				.includeHypervolume()
				.includeInvertedGenerationalDistance()
//				.withIdealPoint(minBound)
//				.withReferencePoint(maxBound)
				.showStatisticalSignificance()
				.showAll();
		
		//run each algorithm for 30 seeds
//		for (String algorithm : algorithms) {
//			analyzer.addAll(algorithm, 
//					executor.withAlgorithm(algorithm).runSeeds(5));
//		}
		
		for (String algorithm : algorithms) {
			System.out.println(algorithm);
			for (int j = 0; j < 30; j++) {
				String alg= new String(algorithm);
				long startTime=System.currentTimeMillis();
				PRNG.setSeed(j+1);
				System.out.println(j+1);
                analyzer.add(alg, executor.withAlgorithm(algorithm).run());
                //analyzer.addAll(algorithm, executor.withAlgorithm(algorithm).runSeeds(2));
    			long endTime=System.currentTimeMillis();
    			System.out.println("running time:" + (endTime-startTime)+"ms");
                //Accumulator accumulator = instrumenter.getLastAccumulator();
 
                //System.out.format(" %-8.4f%n ", accumulator.get("Elapsed Time", accumulator.size("NFE")-1));
                //System.out.format("%-8.4f%n ", accumulator.get("Elapsed Time", 0));

                // print the runtime dynamics
                //System.out.format("  NFE    Time  Hypervolume  G-Distance%n");
                //System.out.format("  NFE    Time   %n");
//               for (int i=0; i<accumulator.size("NFE"); i++) {
//            	   System.out.format("%5d    %-8.4f %-8.4f %-8.4f%n",
//   						accumulator.get("NFE", i),
//   						//accumulator.get("Elapsed Time", i),
//   						accumulator.get("Hypervolume", i),
//   						accumulator.get("GenerationalDistance", i)
//            			);
//                }
			}
		}

		//print the results

		analyzer.printAnalysis();
		//Accumulator accumulator = instrumenter.getLastAccumulator();
		
		// print the runtime dynamics
//		System.out.format("  NFE    Time      Generational Distance%n");
//		
//		for (int i=0; i<accumulator.size("NFE"); i++) {
//			System.out.format("%5d    %-8.4f  %-8.4f%n",
//					accumulator.get("NFE", i),
//					accumulator.get("Elapsed Time", i),
//					accumulator.get("GenerationalDistance", i));
//		}
	}

}
