package org.moeaframework.core.fitness;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.moeaframework.core.FitnessEvaluator;
import org.moeaframework.core.Population;
import org.moeaframework.core.Solution;

//reverse Chebyshev distance is calculated 


public class RchDistanceFitnessEvaluator implements FitnessEvaluator{
	
	
	/**
	 * Constructs a new crowding distance fitness evaluator.
	 */
	public RchDistanceFitnessEvaluator() {
		super();
	}

	@Override
	public void evaluate(Population population) {
		if (population.size() <= 2) {
			for (Solution solution : population) {
				solution.setAttribute(FITNESS_ATTRIBUTE, 0.0);
			}
		} else {
			
			List<Solution> solutions = copy(population);
			int n = population.size();
			
			double[][] Rchdis= new double[n][n];
			calculateRchDis(solutions, Rchdis);
			
			double totalGI = calculateGI(n, n, Rchdis);
			
			for (int i = 0; i < n; i++) {
			
				double gi = calculateGI(n, i, Rchdis);
				population.get(i).setAttribute(FITNESS_ATTRIBUTE, totalGI - gi);
			}
		}
	}
	

	private void calculateRchDis(List<Solution> solutions, double[][] RchDis) {
		int n = solutions.size();
		int length = solutions.get(0).getNumberOfObjectives();
		double distance = 0;
		for (int i = 0; i < n-1; i++) {
			for (int j = i+1; j < n; j++) {
				
				distance = Math.abs(solutions.get(i).getObjective(0)-solutions.get(j).getObjective(0));  
				
				for (int f = 1; f < length; f++) 
				{
					double temp = Math.abs(solutions.get(i).getObjective(f)-solutions.get(j).getObjective(f)) ;
					if (temp < distance)
					{
						distance = temp;
					}
				}

				RchDis[i][j] = distance;
				RchDis[j][i] = distance;
			}
		}
	}

	private double calculateGI(int n, int number, double[][] RchDis) {
		double givalue = 1;
		
		if (number == n)
		{
			for (int i = 0; i < n; i++) {
				double[] onesolution = new double[n-1];
				int j = 0;
				while (j < i)
				{
					onesolution[j] = RchDis[i][j];
					j++;
				}
				while (j < n-1)
				{
					onesolution[j] = RchDis[i][j+1];
					j++;
				}
				
				Arrays.sort(onesolution);
				givalue = givalue * onesolution[0];
			}
		}
		else
		{
			double[][] RchDis1 = new double[n-1][n-1];
			int jrow = 0;
			int jcolumn = 0;
			while (jrow < number)
			{
				jcolumn = 0;
				while (jcolumn < number)
				{
					RchDis1[jrow][jcolumn] = RchDis[jrow][jcolumn];
					jcolumn++;
				}
				while (jcolumn < n-1)
				{
					RchDis1[jrow][jcolumn] = RchDis[jrow][jcolumn+1];
					jcolumn++;
				}
				jrow++;
			}
				
			while (jrow < n-1)
			{
				jcolumn = 0;
				while (jcolumn < number)
				{
					RchDis1[jrow][jcolumn] = RchDis[jrow+1][jcolumn];
					jcolumn++;
				}
				while (jcolumn < n-1)
				{
					RchDis1[jrow][jcolumn] = RchDis[jrow+1][jcolumn+1];
					jcolumn++;
				}
				jrow++;
			}	
				


			for (int i = 0; i < n-1; i++) {
				double[] onesolution = new double[n-2];
				int j = 0;
				while (j < i)
				{
					onesolution[j] = RchDis1[i][j];
					j++;
				}
				while (j < n-2)
				{
					onesolution[j] = RchDis1[i][j+1];
					j++;
				}
				
				Arrays.sort(onesolution);
				givalue = givalue * onesolution[0];
			}
			
			
		}

		return givalue;
	}

	/**
	 * Returns a copy of the population.  The fast non-dominated sorting
	 * routine reorders solutions in the population, so creating a copy allows
	 * the original population to remain unchanged.  
	 * 
	 * @param population the original population
	 * @return a copy of the population
	 */
	private List<Solution> copy(Population population) {
		List<Solution> pool = new ArrayList<Solution>();
		
		for (Solution solution : population) {
			pool.add(solution);
		}
		
		return pool;
	}
	
	
	@Override
	public boolean areLargerValuesPreferred() {
		return true;
	}



}
